import typing
import datetime

import pydantic

import schemaless.config
import schemaless.http_client
import schemaless.bases
import schemaless.rid

DOMAIN_BASE_PATH = "/api/v1/domain"


class DomainModel(pydantic.BaseModel):
    rid: schemaless.rid.RID
    name: str
    description: str
    json_schema: typing.Union[typing.Dict[str, typing.Any], None] = None
    created_at: int
    updated_at: int
    deleted_at: typing.Union[int, None] = None

    def get_created_at_datetime(self) -> datetime.datetime:
        return datetime.datetime.fromtimestamp(self.created_at)

    def get_updated_at_datetime(self) -> datetime.datetime:
        return datetime.datetime.fromtimestamp(self.updated_at)

    def get_deleted_at_datetime(self):
        return datetime.datetime.fromtimestamp(self.deleted_at)

    @classmethod
    def from_api_response(cls, response: dict) -> "DomainModel":
        return cls(
            **response,
        )


class CreateDomainBody(pydantic.BaseModel):
    rid: typing.Union[str, None] = None
    name: str
    description: str
    json_schema: typing.Union[typing.Dict[str, typing.Any], None] = None


class Domain(schemaless.bases.Handler):
    def __init__(self, http_client: schemaless.bases.BaseHTTP):
        self.http_client = http_client

    @classmethod
    def from_config(cls, config: schemaless.config.Config) -> "Domain":
        return cls(
            http_client=schemaless.http_client.HTTP.from_config(config),
        )

    async def create(self, domain: CreateDomainBody) -> DomainModel:
        response = await self.http_client.post(
            url=DOMAIN_BASE_PATH + "/add",
            json=domain.model_dump(),
        )

        return response.parse_to_model(DomainModel)

    def search(self):
        pass

    def get(self):
        pass

    def list(self):
        pass

    def delete(self):
        pass

    def update(self):
        pass



import os


class Config:
    def __init__(self, api_key: str, base_url: str):
        self.api_key = api_key
        self.base_url = base_url

    @classmethod
    def from_env(cls) -> "Config":
        return cls(
            api_key=os.environ["SCHEMALESS_API_KEY"],
            base_url=os.environ["SCHEMALESS_BASE_URL"],
        )

    def get_api_key(self) -> str:
        return self.api_key

    def get_base_url(self) -> str:
        return self.base_url



import enum
import typing
import datetime

import cloudevents.pydantic.v2
import pydantic

import schemaless.rid
import schemaless.bases
import schemaless.config
import schemaless.http_client
import schemaless.delta


class WebhookModel(pydantic.BaseModel):
    rid: schemaless.rid.RID
    alias: typing.Union[str, None] = None
    name_prefix: typing.Union[str, None] = None
    domain_rid: schemaless.rid.RID
    pattern: str
    on_operations: typing.List[schemaless.delta.Operation]
    url: str
    created_at: int
    modified_at: int
    deleted_at: typing.Union[int, None] = None

    def get_created_at_datetime(self) -> datetime.datetime:
        return datetime.datetime.fromtimestamp(self.created_at)

    def get_updated_at_datetime(self) -> datetime.datetime:
        return datetime.datetime.fromtimestamp(self.modified_at)

    def get_deleted_at_datetime(self):
        return datetime.datetime.fromtimestamp(self.deleted_at)

    @classmethod
    def from_api_response(cls, response: dict) -> "WebhookModel":
        return cls(
            **response,
        )


class RegisterWebhookBody(pydantic.BaseModel):
    domain_rid: schemaless.rid.RID
    pattern: str
    on_operations: typing.List[schemaless.delta.Operation]
    url: str
    alias: typing.Union[str, None] = None
    name_prefix: typing.Union[str, None] = None

    model_config = pydantic.ConfigDict(use_enum_values=True)


class WebhookHistoryStatus(enum.Enum):
    Success = "Success"
    Error = "Error"
    Fail = "Fail"


class HistoryMetadata(pydantic.BaseModel):
    domain_rid: schemaless.rid.RID
    resource_rid: schemaless.rid.RID
    webhook_rid: schemaless.rid.RID


class History(pydantic.BaseModel):
    rid: schemaless.rid.RID
    metadata: HistoryMetadata
    event: cloudevents.pydantic.v2.CloudEvent
    created_at: int
    status: WebhookHistoryStatus
    message: typing.Union[str, None] = None


class HistoryResponse(pydantic.BaseModel):
    history: typing.List[History] = []


class EventHistoryFilters(pydantic.BaseModel):
    domain_rid: typing.Union[schemaless.rid.RID, None] = None
    history_rid: typing.Union[schemaless.rid.RID, None] = None
    resource_rid: typing.Union[schemaless.rid.RID, None] = None

    def to_request_params(self) -> dict:
        base = {}
        if self.domain_rid is not None:
            base["domain_rid"] = self.domain_rid.to_string()
        if self.history_rid is not None:
            base["history_rid"] = self.history_rid.to_string()
        if self.resource_rid is not None:
            base["resource_rid"] = self.resource_rid.to_string()
        return base


class WebhookHistory(pydantic.BaseModel):
    domain_rid: schemaless.rid.RID


class Webhook(schemaless.bases.Handler):
    def __init__(self, http_client: schemaless.bases.BaseHTTP):
        self.http_client = http_client

    @classmethod
    def from_config(cls, config: schemaless.config.Config) -> "Webhook":
        return cls(
            http_client=schemaless.http_client.HTTP.from_config(config),
        )

    async def register(self, webhook: RegisterWebhookBody) -> WebhookModel:
        response = await self.http_client.post(
            url="/api/v1/webhook/register",
            json=RegisterWebhookBody(
                domain_rid=webhook.domain_rid,
                pattern=webhook.pattern,
                on_operations=webhook.on_operations,
                url=webhook.url,
                alias=webhook.alias,
                name_prefix=webhook.name_prefix,
            ).model_dump(),
        )

        return response.parse_to_model(WebhookModel)

    async def history(self, filters: EventHistoryFilters) -> HistoryResponse:
        response = await self.http_client.get(
            url="/api/v1/webhook/history",
            params=filters.to_request_params(),
        )

        return response.parse_to_model(HistoryResponse)



import enum
import typing

import pydantic

"""
#[derive(Debug, PartialEq, Clone, Serialize, Deserialize)]
pub enum Operation {
    Add,
    Change,
    Delete,
}
"""


class Operation(enum.Enum):
    ADD = "Add"
    CHANGE = "Change"
    DELETE = "Delete"


class DeltaModel(pydantic.BaseModel):
    operation: Operation
    path: str
    old_value: typing.Union[typing.Any, None] = None
    new_value: typing.Union[typing.Any, None] = None
    hash: str

    @classmethod
    def from_api_response(cls, response: dict) -> "DeltaModel":
        return cls(
            **response,
        )



import dataclasses
import typing
import uuid

import pydantic


def id_generator() -> str:
    return str(uuid.uuid4())


class RID(pydantic.BaseModel):
    namespace: str
    domain: str
    id: typing.Optional[str] = pydantic.Field(default_factory=id_generator)

    def to_string(self) -> str:
        return f"{self.namespace}:{self.domain}:{self.id}"
